"use strict";
/**
 * @license
 * Copyright 2018 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var engine_1 = require("../engine");
var globals_1 = require("../globals");
var ops_1 = require("../ops/ops");
var serialization_1 = require("../serialization");
var optimizer_1 = require("./optimizer");
/** @doclink Optimizer */
var RMSPropOptimizer = /** @class */ (function (_super) {
    __extends(RMSPropOptimizer, _super);
    function RMSPropOptimizer(learningRate, decay, momentum, epsilon, centered) {
        if (decay === void 0) { decay = 0.9; }
        if (momentum === void 0) { momentum = 0.0; }
        if (epsilon === void 0) { epsilon = null; }
        if (centered === void 0) { centered = false; }
        var _this = _super.call(this) || this;
        _this.learningRate = learningRate;
        _this.decay = decay;
        _this.momentum = momentum;
        _this.epsilon = epsilon;
        _this.accumulatedMeanSquares = [];
        _this.accumulatedMoments = [];
        _this.accumulatedMeanGrads = [];
        _this.centered = centered;
        if (epsilon == null) {
            _this.epsilon = engine_1.ENGINE.backend.epsilon();
        }
        return _this;
    }
    RMSPropOptimizer.prototype.applyGradients = function (variableGradients) {
        var _this = this;
        var variableNames = Array.isArray(variableGradients) ?
            variableGradients.map(function (item) { return item.name; }) :
            Object.keys(variableGradients);
        var _loop_1 = function (i) {
            var name_1 = variableNames[i];
            var value = engine_1.ENGINE.registeredVariables[name_1];
            var trainable = false;
            if (this_1.accumulatedMeanSquares[i] == null) {
                this_1.accumulatedMeanSquares[i] = {
                    name: name_1 + "/rms",
                    variable: globals_1.tidy(function () { return ops_1.zerosLike(value).variable(trainable); })
                };
            }
            if (this_1.accumulatedMoments[i] == null) {
                this_1.accumulatedMoments[i] = {
                    name: name_1 + "/momentum",
                    variable: globals_1.tidy(function () { return ops_1.zerosLike(value).variable(trainable); })
                };
            }
            if (this_1.accumulatedMeanGrads[i] == null && this_1.centered) {
                this_1.accumulatedMeanGrads[i] = {
                    name: name_1 + "/mg",
                    variable: globals_1.tidy(function () { return ops_1.zerosLike(value).variable(trainable); })
                };
            }
            var accumulatedMeanSquare = this_1.accumulatedMeanSquares[i];
            var accumulatedMeanGrad = this_1.accumulatedMeanGrads[i];
            var accumulatedMoments = this_1.accumulatedMoments[i];
            var gradient = Array.isArray(variableGradients) ?
                variableGradients[i].tensor : variableGradients[name_1];
            globals_1.tidy(function () {
                var newAccumulatedMeanSquare = accumulatedMeanSquare.variable.mul(_this.decay)
                    .add(gradient.square().mul(1 - _this.decay));
                if (_this.centered) {
                    // Centered gradient
                    var newAccumulatedMeanGrad = accumulatedMeanGrad.variable
                        .mul(_this.decay).add(gradient.mul(1 - _this.decay));
                    var newAccumulatedMoments = accumulatedMoments.variable
                        .mul(_this.momentum)
                        .add(gradient.mul(_this.learningRate)
                        .div(newAccumulatedMeanSquare
                        .sub(newAccumulatedMeanGrad.square().add(_this.epsilon))
                        .sqrt()));
                    _this.accumulatedMeanSquares[i].variable.assign(newAccumulatedMeanSquare);
                    _this.accumulatedMeanGrads[i].variable.assign(newAccumulatedMeanGrad);
                    _this.accumulatedMoments[i].variable.assign(newAccumulatedMoments);
                    var newValue = value.sub(newAccumulatedMoments);
                    value.assign(newValue);
                }
                else {
                    // Plain gradient
                    var newAccumulatedMeanSquare_1 = accumulatedMeanSquare.variable.mul(_this.decay)
                        .add(gradient.square().mul(1 - _this.decay));
                    var newAccumulatedMoments = accumulatedMoments.variable.mul(_this.momentum)
                        .add(gradient.mul(_this.learningRate)
                        .div(newAccumulatedMeanSquare_1.add(_this.epsilon)
                        .sqrt()));
                    _this.accumulatedMeanSquares[i].variable.assign(newAccumulatedMeanSquare_1);
                    _this.accumulatedMoments[i].variable.assign(newAccumulatedMoments);
                    var newValue = value.sub(newAccumulatedMoments);
                    value.assign(newValue);
                }
            });
        };
        var this_1 = this;
        for (var i = 0; i < variableNames.length; ++i) {
            _loop_1(i);
        }
    };
    RMSPropOptimizer.prototype.dispose = function () {
        if (this.accumulatedMeanSquares != null) {
            globals_1.dispose(this.accumulatedMeanSquares.map(function (v) { return v.variable; }));
        }
        if (this.accumulatedMeanGrads != null && this.centered) {
            globals_1.dispose(this.accumulatedMeanGrads.map(function (v) { return v.variable; }));
        }
        if (this.accumulatedMoments != null) {
            globals_1.dispose(this.accumulatedMoments.map(function (v) { return v.variable; }));
        }
    };
    RMSPropOptimizer.prototype.getWeights = function () {
        // Order matters for Python compatibility.
        var namedVariables = this.accumulatedMeanSquares.concat(this.accumulatedMoments);
        if (this.centered) {
            namedVariables.push.apply(namedVariables, this.accumulatedMeanGrads);
        }
        return namedVariables.map(function (v) { return ({ name: v.name, tensor: v.variable }); });
    };
    RMSPropOptimizer.prototype.setWeights = function (weightValues) {
        var variableCount = this.centered ? weightValues.length / 3 : weightValues.length / 2;
        var trainable = false;
        this.accumulatedMeanSquares = weightValues.slice(0, variableCount).map(function (v) { return ({ name: v.name, variable: v.tensor.variable(trainable) }); });
        this.accumulatedMoments =
            weightValues.slice(variableCount, variableCount * 2).map(function (v) { return ({ name: v.name, variable: v.tensor.variable(trainable) }); });
        if (this.centered) {
            this.accumulatedMeanGrads =
                weightValues.slice(variableCount * 2, variableCount * 3).map(function (v) { return ({ name: v.name, variable: v.tensor.variable(trainable) }); });
        }
    };
    RMSPropOptimizer.prototype.getConfig = function () {
        return {
            'learningRate': this.learningRate,
            'decay': this.decay,
            'momentum': this.momentum,
            'epsilon': this.epsilon,
            'centered': this.centered
        };
    };
    /** @nocollapse */
    RMSPropOptimizer.fromConfig = function (cls, config) {
        return new cls(config['learningRate'], config['decay'], config['momentum'], config['epsilon'], config['centered']);
    };
    /** @nocollapse */
    RMSPropOptimizer.className = 'RMSProp';
    return RMSPropOptimizer;
}(optimizer_1.Optimizer));
exports.RMSPropOptimizer = RMSPropOptimizer;
serialization_1.registerClass(RMSPropOptimizer);
//# sourceMappingURL=rmsprop_optimizer.js.map